(***********************************************************************)
(*                                                                     *)
(*                          HEVEA                                      *)
(*                                                                     *)
(*  Luc Maranget, projet PARA, INRIA Rocquencourt                      *)
(*                                                                     *)
(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

exception Error of string
val put_from_file : string -> (string -> unit) -> unit
val copy_from_lib : string -> string -> unit
val copy_from_lib_to_dir : string -> string option -> string -> unit
val rename : string -> string -> unit
val remove : string -> unit
